@@ -360,6 +360,7 @@ def getHyperparameters(configFile, videoName, videoPath, argv):
   hyperparameters["outputValidationVideoContrastImprovementQuartile"] = getConfig(config, "outputValidationVideoContrastImprovementQuartile", videoPath)
   
   hyperparameters["videoFPS"] = getConfig(config, "videoFPS", videoPath)
+  hyperparameters["videoPixelSize"] = getConfig(config, "videoPixelSize", videoPath)
   
   hyperparameters["groupOfMultipleSameSizeAndShapeEquallySpacedWells"] = getConfig(config, "groupOfMultipleSameSizeAndShapeEquallySpacedWells", videoPath)
   
@@ -367,6 +368,27 @@ def getHyperparameters(configFile, videoName, videoPath, argv):
   
   hyperparameters["copyOriginalVideoToOutputFolderForValidation"] = getConfig(config, "copyOriginalVideoToOutputFolderForValidation", videoPath)
   
+  hyperparameters["postProcessRemoveLowProbabilityDetection"] = getConfig(config, "postProcessRemoveLowProbabilityDetection", videoPath)
+  hyperparameters["postProcessLowProbabilityDetectionThreshold"] = getConfig(config, "postProcessLowProbabilityDetectionThreshold", videoPath)
+  hyperparameters["postProcessRemovePointsOnBordersMargin"] = getConfig(config, "postProcessRemovePointsOnBordersMargin", videoPath)
+  hyperparameters["postProcessRemovePointsAwayFromMainTrajectory"] = getConfig(config, "postProcessRemovePointsAwayFromMainTrajectory", videoPath)
+  hyperparameters["postProcessRemovePointsAwayFromMainTrajectoryThreshold"] = getConfig(config, "postProcessRemovePointsAwayFromMainTrajectoryThreshold", videoPath)
+  
+  hyperparameters["reloadWellPositionsFromFileInZZoutputIfItExistSaveInItOtherwise"] = getConfig(config, "reloadWellPositionsFromFileInZZoutputIfItExistSaveInItOtherwise", videoPath)
+  
+  hyperparameters["tailAnglesHeatMap"] = getConfig(config, "tailAnglesHeatMap", videoPath)
+  hyperparameters["tailAnglesHeatMapNbPointsToTakeIntoAccount"] = getConfig(config, "tailAnglesHeatMapNbPointsToTakeIntoAccount", videoPath)
+  
+  hyperparameters["setBackgroundToImageMedian"] = getConfig(config, "setBackgroundToImageMedian", videoPath)
+  
+  hyperparameters["backgroundSubtractorKNN"] = getConfig(config, "backgroundSubtractorKNN", videoPath)
+  
+  hyperparameters["postProcessLowProbabilityDetectionPercentOfMaximum"] = getConfig(config, "postProcessLowProbabilityDetectionPercentOfMaximum", videoPath)
+  
+  
+  if hyperparameters["tailAnglesHeatMap"]:
+    hyperparameters["calculateAllTailAngles"] = 1
+  
   
   if len(argv) > 5 and not(argv[0] == "getTailExtremityFirstFrame"):
     i = 5
